/*
 * Copyright (C) 2019-2024 B.R.S. Roso.
 * 
 * Musp is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License version 3 as published by the Free
 * Software Foundation.
 * 
 * This program is distributed in the hope that it be useful but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Musp.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <stdlib.h>
#include <string.h>
#include "muspd.h"

#define LI_INIT_SIZE 16

char **li;
size_t li_len;
static size_t li_size;

int li_init()
{
	li_size = LI_INIT_SIZE;
	li = malloc((sizeof *li) * li_size);
	if (li == NULL) {
		perrf_errno("failed to allocate memory");
		return 1;
	}
	li_len = 0;

	return 0;
}

int li_incr()
{
	const size_t new_li_size = li_size + li_size / 2 + 1;
	void * const p = realloc(li, (sizeof *li) * new_li_size);
	if (p == NULL) {
		perrf_errno("failed to allocate memory");
		return 1;
	}
	li = p;
	li_size = new_li_size;
	
	return 0;
}

#if 0
int li_clear()
{
	size_t i;
	for (i = 0; i < li_len; i++)
		free(li[i]);
	const size_t new_li_size = LI_INIT_SIZE;
	void * const p = realloc(li, (sizeof *li) * new_li_size);
	if (p == NULL)
		return 1;
	li = p;
	li_size = new_li_size;
	li_len = 0;

	return 0;
}
#endif

int li_add(const char *str)
{
	if (li_size == li_len)
		if (li_incr())
			goto err;
	li[li_len++] = strdup(str);

	return 0;

err:
	perrf("failed to add to list");
	return 1;
}
